=== modified file 'NEWS'
--- NEWS	2010-06-17 11:52:13 +0000
+++ NEWS	2010-06-17 14:06:36 +0000
@@ -66,14 +66,20 @@
   which previously caused "SyntaxError: No command for line".
   (Martin Pool)
 
-* ``walkdirs`` now raises a useful message when the filenames are not using
-  the filesystem encoding. (Eric Moritz, #488519)
+* Show unicode filenames in diff headers using terminal encoding. 
+  (Alexander Belchenko, Bug #382699)
+  NOTE for Windows users: If user need to save diff to file then user need to
+  change encoding of the terminal to ANSI encoding with command ``chcp XXX``
+  (e.g. ``chcp 1251`` for Russian Windows).
 
 * URL displayed for use with ``break-lock`` when smart server sees lock
   contention are now valid. Default timeout for lock contention retry is
   now 30 seconds instead of 300 seconds.
   (Parth Malwankar, #250451)
 
+* ``walkdirs`` now raises a useful message when the filenames are not using
+  the filesystem encoding. (Eric Moritz, #488519)
+
 Improvements
 ************
 

=== modified file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	2010-06-07 02:38:04 +0000
+++ bzrlib/builtins.py	2010-06-17 08:53:15 +0000
@@ -1970,11 +1970,15 @@
          old_branch, new_branch,
          specific_files, extra_trees) = get_trees_and_branches_to_diff_locked(
             file_list, revision, old, new, self.add_cleanup, apply_view=True)
+        # GNU diff on Windows uses ANSI encoding for filenames
+        path_encoding = osutils.get_diff_header_encoding()
         return show_diff_trees(old_tree, new_tree, sys.stdout,
                                specific_files=specific_files,
                                external_diff_options=diff_options,
                                old_label=old_label, new_label=new_label,
-                               extra_trees=extra_trees, using=using,
+                               extra_trees=extra_trees,
+                               path_encoding=path_encoding,
+                               using=using,
                                format_cls=format)
 
 
@@ -3892,8 +3896,10 @@
     def _do_preview(self, merger):
         from bzrlib.diff import show_diff_trees
         result_tree = self._get_preview(merger)
+        path_encoding = osutils.get_diff_header_encoding()
         show_diff_trees(merger.this_tree, result_tree, self.outf,
-                        old_label='', new_label='')
+                        old_label='', new_label='',
+                        path_encoding=path_encoding)
 
     def _do_merge(self, merger, change_reporter, allow_pending, verified):
         merger.change_reporter = change_reporter

=== modified file 'bzrlib/diff.py'
--- bzrlib/diff.py	2010-05-25 17:27:52 +0000
+++ bzrlib/diff.py	2010-05-26 15:58:08 +0000
@@ -99,8 +99,8 @@
     if sequence_matcher is None:
         sequence_matcher = patiencediff.PatienceSequenceMatcher
     ud = patiencediff.unified_diff(oldlines, newlines,
-                      fromfile=old_filename.encode(path_encoding),
-                      tofile=new_filename.encode(path_encoding),
+                      fromfile=old_filename.encode(path_encoding, 'replace'),
+                      tofile=new_filename.encode(path_encoding, 'replace'),
                       sequencematcher=sequence_matcher)
 
     ud = list(ud)
@@ -713,11 +713,11 @@
             from_text = _get_text(self.old_tree, from_file_id, from_path)
             to_text = _get_text(self.new_tree, to_file_id, to_path)
             self.text_differ(from_label, from_text, to_label, to_text,
-                             self.to_file)
+                             self.to_file, path_encoding=self.path_encoding)
         except errors.BinaryFile:
             self.to_file.write(
                   ("Binary files %s and %s differ\n" %
-                  (from_label, to_label)).encode(self.path_encoding))
+                  (from_label, to_label)).encode(self.path_encoding,'replace'))
         return self.CHANGED
 
 
@@ -920,7 +920,10 @@
             extra_factories = []
         if external_diff_options:
             opts = external_diff_options.split()
-            def diff_file(olab, olines, nlab, nlines, to_file):
+            def diff_file(olab, olines, nlab, nlines, to_file, path_encoding=None):
+                """:param path_encoding: not used but required
+                        to match the signature of internal_diff.
+                """
                 external_diff(olab, olines, nlab, nlines, to_file, opts)
         else:
             diff_file = internal_diff

=== modified file 'bzrlib/log.py'
--- bzrlib/log.py	2010-06-08 09:50:27 +0000
+++ bzrlib/log.py	2010-06-17 08:53:15 +0000
@@ -70,6 +70,7 @@
     diff,
     errors,
     foreign,
+    osutils,
     repository as _mod_repository,
     revision as _mod_revision,
     revisionspec,
@@ -432,8 +433,9 @@
         else:
             specific_files = None
         s = StringIO()
+        path_encoding = osutils.get_diff_header_encoding()
         diff.show_diff_trees(tree_1, tree_2, s, specific_files, old_label='',
-            new_label='')
+            new_label='', path_encoding=path_encoding)
         return s.getvalue()
 
     def _create_log_revision_iterator(self):

=== modified file 'bzrlib/osutils.py'
--- bzrlib/osutils.py	2010-06-17 07:37:32 +0000
+++ bzrlib/osutils.py	2010-06-17 14:06:36 +0000
@@ -1968,6 +1968,10 @@
     return user_encoding
 
 
+def get_diff_header_encoding():
+    return get_terminal_encoding()
+
+
 def get_host_name():
     """Return the current unicode host name.
 

=== modified file 'bzrlib/shelf_ui.py'
--- bzrlib/shelf_ui.py	2010-01-15 05:33:28 +0000
+++ bzrlib/shelf_ui.py	2010-05-26 15:58:08 +0000
@@ -241,7 +241,8 @@
             new_tree = self.work_tree
         old_path = old_tree.id2path(file_id)
         new_path = new_tree.id2path(file_id)
-        text_differ = diff.DiffText(old_tree, new_tree, diff_file)
+        text_differ = diff.DiffText(old_tree, new_tree, diff_file,
+            path_encoding=osutils.get_terminal_encoding())
         patch = text_differ.diff(file_id, old_path, new_path, 'file', 'file')
         diff_file.seek(0)
         return patches.parse_patch(diff_file)
@@ -493,7 +494,9 @@
         new_tree = tt.get_preview_tree()
         if self.write_diff_to is None:
             self.write_diff_to = ui.ui_factory.make_output_stream()
-        diff.show_diff_trees(merger.this_tree, new_tree, self.write_diff_to)
+        path_encoding = osutils.get_diff_header_encoding()
+        diff.show_diff_trees(merger.this_tree, new_tree, self.write_diff_to,
+            path_encoding=path_encoding)
         tt.finalize()
 
     def show_changes(self, merger):

=== modified file 'bzrlib/tests/test_diff.py'
--- bzrlib/tests/test_diff.py	2010-05-20 08:08:20 +0000
+++ bzrlib/tests/test_diff.py	2010-05-26 15:58:08 +0000
@@ -34,6 +34,7 @@
     )
 from bzrlib.symbol_versioning import deprecated_in
 from bzrlib.tests import features
+from bzrlib.tests.blackbox.test_diff import subst_dates
 
 
 class _AttribFeature(tests.Feature):
@@ -521,7 +522,6 @@
         self.assertNotContainsRe(d, r"file 'e'")
         self.assertNotContainsRe(d, r"file 'f'")
 
-
     def test_binary_unicode_filenames(self):
         """Test that contents of files are *not* encoded in UTF-8 when there
         is a binary file in the diff.
@@ -580,6 +580,49 @@
         self.assertContainsRe(d, "=== modified file 'mod_%s'"%autf8)
         self.assertContainsRe(d, "=== removed file 'del_%s'"%autf8)
 
+    def test_unicode_filename_path_encoding(self):
+        """Test for bug #382699: unicode filenames on Windows should be shown
+        in user encoding.
+        """
+        self.requireFeature(tests.UnicodeFilenameFeature)
+        # The word 'test' in Russian
+        _russian_test = u'\u0422\u0435\u0441\u0442'
+        directory = _russian_test + u'/'
+        test_txt = _russian_test + u'.txt'
+        u1234 = u'\u1234.txt'
+
+        tree = self.make_branch_and_tree('.')
+        self.build_tree_contents([
+            (test_txt, 'foo\n'),
+            (u1234, 'foo\n'),
+            (directory, None),
+            ])
+        tree.add([test_txt, u1234, directory])
+
+        sio = StringIO()
+        diff.show_diff_trees(tree.basis_tree(), tree, sio,
+            path_encoding='cp1251')
+
+        output = subst_dates(sio.getvalue())
+        shouldbe = ('''\
+=== added directory '%(directory)s'
+=== added file '%(test_txt)s'
+--- a/%(test_txt)s\tYYYY-MM-DD HH:MM:SS +ZZZZ
++++ b/%(test_txt)s\tYYYY-MM-DD HH:MM:SS +ZZZZ
+@@ -0,0 +1,1 @@
++foo
+
+=== added file '?.txt'
+--- a/?.txt\tYYYY-MM-DD HH:MM:SS +ZZZZ
++++ b/?.txt\tYYYY-MM-DD HH:MM:SS +ZZZZ
+@@ -0,0 +1,1 @@
++foo
+
+''' % {'directory': _russian_test.encode('cp1251'),
+       'test_txt': test_txt.encode('cp1251'),
+      })
+        self.assertEqualDiff(output, shouldbe)
+
 
 class DiffWasIs(diff.DiffPath):
 

